@@ -14,7 +14,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.01';
+$VERSION = '1.02';
 
 sub ProcessBitStream($$$);
 
@@ -59,10 +59,9 @@ sub ProcessBitStream($$$);
     Duration => {
         Require => {
             0 => 'FLAC:SampleRate',
-            1 => 'FLAC:Channels',
-            2 => 'FLAC:TotalSamples',
+            1 => 'FLAC:TotalSamples',
         },
-        ValueConv => '$val[0] and $val[1] and $val[2] ? $val[2] / ($val[0] * $val[1]) : undef',
+        ValueConv => '$val[0] and $val[1] ? $val[1] / $val[0] : undef',
         PrintConv => 'ConvertDuration($val)',
     },
 );
@@ -158,7 +157,7 @@ sub ProcessFLAC($$)
     my ($exifTool, $dirInfo) = @_;
 
     # must first check for leading/trailing ID3 information
-    unless ($exifTool->{DONE_ID3}) {
+    unless ($exifTool->{DoneID3}) {
         require Image::ExifTool::ID3;
         Image::ExifTool::ID3::ProcessID3($exifTool, $dirInfo) and return 1;
     }
@@ -169,7 +168,7 @@ sub ProcessFLAC($$)
 
     # check FLAC signature
     $raf->Read($buff, 4) == 4 and $buff eq 'fLaC' or return 0;
-    $exifTool->SetFileType() unless $exifTool->{VALUE}->{FileType};
+    $exifTool->SetFileType();
     SetByteOrder('MM');
     my $tagTablePtr = GetTagTable('Image::ExifTool::FLAC::Main');
     for (;;) {
