@@ -13,7 +13,7 @@ use vars qw($VERSION);
 use Image::ExifTool;    # only for FinishTiffDump()
 use Image::ExifTool::HTML qw(EscapeHTML);
 
-$VERSION = '1.23';
+$VERSION = '1.27';
 
 sub DumpTable($$$;$$$$$);
 sub Open($$$;@);
@@ -268,7 +268,9 @@ sub Add($$$$;$$)
 #         2) data pointer, 3) data position, 4) output file or scalar reference,
 #         5) limit level (1-3), 6) title
 # Returns: non-zero if useful output was generated,
-#          or -1 on error loading data and ERROR is set to offending data name
+#          or -1 on error loading data and "ERROR" is set to offending data name
+# Note: The "Error" member may be set externally to print a specific error
+#       message instead of doing the dump.
 sub Print($$;$$$$$)
 {
     local $_;
@@ -280,7 +282,6 @@ sub Print($$;$$$$$)
     $title = 'HtmlDump' unless $title;
     $level or $level = 0;
     my $tell = $raf->Tell();
-    my @starts = sort { $a <=> $b } keys %$block;
     my $pos = 0;
     my $dataEnd = $dataPos + ($dataPt ? length($$dataPt) : 0);
     # initialize member variables
@@ -306,7 +307,9 @@ sub Print($$;$$$$$)
     $bkgStart = $bkgEnd = 0;
     $bkgSpan = '';
     my $index = 0;  # initialize tooltip index
-    my (@names, $wasUnused);
+    my (@names, $wasUnused, @starts);
+    # only do dump if we didn't have a serious error
+    @starts = sort { $a <=> $b } keys %$block unless $$self{Error};
     for ($i=0; $i<@starts; ++$i) {
         my $start = $starts[$i];
         my $parmList = $$block{$start};
@@ -314,10 +317,19 @@ sub Print($$;$$$$$)
         if ($len > 0 and not $wasUnused) {
             # we have an unused bytes before this data block
             --$i;           # dump the data block next time around
-            $start = $pos;  # dump the unused bytes now
+            # split unused data into 2 blocks if it spans end of makernotes
+            if (not defined $wasUnused and $bkgEnd and
+                $pos < $bkgEnd + $dataPos and $pos + $len > $bkgEnd + $dataPos)
+            {
+                $start = $pos;
+                $len = $bkgEnd + $dataPos - $pos;
+                $wasUnused = 0;
+            } else {
+                $start = $pos;  # dump the unused bytes now
+                $wasUnused = 1; # avoid re-dumping unused bytes if we get a read error
+            }
             my $str = ($len > 1) ? "unused $len bytes" : 'pad byte';
             $parmList = [ [ $len, "[$str]", undef, 0x108 ] ];
-            $wasUnused = 1; # avoid re-dumping unused bytes if we get a read error
         } else {
             undef $wasUnused;
         }
@@ -340,7 +352,8 @@ sub Print($$;$$$$$)
             }
             if ($flag == 4) {
                 $bkgStart = $start - $dataPos;
-                $bkgEnd = $bkgStart + $len;
+                # (allow for nested makernotes data)
+                $bkgEnd = $bkgStart + $len unless $bkgEnd and $bkgEnd > $bkgStart + $len;
                 $bkgSpan = "<span class='$name M'>";
                 push @{$self->{MSpanList}}, $name;
                 next;
@@ -369,7 +382,7 @@ sub Print($$;$$$$$)
                                 # reset $len to the actual length of available data
                                 $raf->Seek(0, 2);
                                 $len = $raf->Tell() - $start;
-                                $tip .= "\nError: Only $len bytes available!" if $tip;
+                                $tip .= "<br>Error: Only $len bytes available!" if $tip;
                                 next;
                             }
                             $buff .= $buf2;
@@ -377,6 +390,8 @@ sub Print($$;$$$$$)
                         }
                     } else {
                         $err = $msg;
+                        $len = length $buff;
+                        $tip .= "<br>Error: Only $len bytes available!" if $tip;
                     }
                 }
                 last;
@@ -423,8 +438,8 @@ sub Print($$;$$$$$)
         delete $$self{TipList};
         $rtnVal = 1;
     } else {
-        Write($outfile, "$title</title></head><body>\n",
-                        "No EXIF or TIFF information found in image\n");
+        my $err = $$self{Error} || 'No EXIF or TIFF information found in image';
+        Write($outfile, "$title</title></head><body>\n$err\n");
         $rtnVal = 0;
     }
     Write($outfile, "</body></html>\n");
@@ -685,13 +700,36 @@ sub FinishTiffDump($$$)
         OtherImageStart   => 'OtherImageLength',
         ImageOffset       => 'ImageByteCount',
         AlphaOffset       => 'AlphaByteCount',
+        MPImageStart      => 'MPImageLength',
     );
 
     # add TIFF data to html dump
     foreach $tag (keys %offsetPair) {
         my $info = $exifTool->GetInfo($tag);
         next unless %$info;
-        foreach $key (%$info) {
+        # Panasonic hack: StripOffsets is not valid for Panasonic RW2 files,
+        # and StripRowBytes is not valid for some RAW images
+        if ($tag eq 'StripOffsets' and $exifTool->{TAG_INFO}{$tag}{PanasonicHack}) {
+            # use RawDataOffset instead if available since it is valid in RW2
+            my $info2 = $exifTool->GetInfo('RawDataOffset');
+            $info2 = $info unless %$info2;
+            my @keys = keys %$info2;
+            my $offset = $$info2{$keys[0]};
+            my $raf = $$exifTool{RAF};
+            # ignore StripByteCounts and assume raw data runs to the end of file
+            if (@keys == 1 and $offset =~ /^\d+$/ and $raf) {
+                my $pos = $raf->Tell();
+                $raf->Seek(0, 2);   # seek to end
+                my $len = $raf->Tell() - $offset;
+                $raf->Seek($pos, 0);
+                if ($len > 0) {
+                    $self->Add($offset, $len, "(Panasonic raw data)", "Size: $len bytes", 0x08);
+                    next;
+                }
+            }
+        }
+        # loop through all offsets tags
+        foreach $key (keys %$info) {
             my $name = Image::ExifTool::GetTagName($key);
             my $grp1 = $exifTool->GetGroup($key, 1);
             my $info2 = $exifTool->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
@@ -699,6 +737,8 @@ sub FinishTiffDump($$$)
             my ($key2) = keys %$info2;
             my $offsets = $$info{$key};
             my $byteCounts = $$info2{$key2};
+            # ignore primary MPImage (this is the whole JPEG)
+            next if $tag eq 'MPImageStart' and $offsets eq '0';
             # (long lists may be SCALAR references)
             my @offsets = split ' ', (ref $offsets ? $$offsets : $offsets);
             my @byteCounts = split ' ', (ref $byteCounts ? $$byteCounts : $byteCounts);
