@@ -20,6 +20,8 @@
 #              12) Ioannis Panagiotopoulos private communication (E-510)
 #              13) Chris Shaw private communication (E-3)
 #              14) Viktor Lushnikov private communication (E-400)
+#              15) Yrjo Rauste private communication (E-30)
+#              16) Godfrey DiGiorgi private communcation (E-P1) + http://forums.dpreview.com/forums/read.asp?message=33187567
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Olympus;
@@ -29,7 +31,7 @@ use vars qw($VERSION);
 use Image::ExifTool::Exif;
 use Image::ExifTool::APP12;
 
-$VERSION = '1.56';
+$VERSION = '1.63';
 
 sub PrintLensInfo($$$);
 
@@ -42,6 +44,7 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
 #
 # Tags 0x0000 through 0x0103 are the same as Konica/Minolta cameras (ref 3)
+# (removed 0x0101-0x0103 because they weren't supported by my samples - PH)
 #
     0x0000 => {
         Name => 'MakerNoteVersion',
@@ -91,42 +94,6 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
         WriteCheck => '$self->CheckImage(\$val)',
         Binary => 1,
     },
-    0x0101 => {
-        Name => 'ColorMode',
-        Writable => 'int16u',
-        PrintConv => {
-            0 => 'Natural color',
-            1 => 'Black&white',
-            2 => 'Vivid color',
-            3 => 'Solarization',
-            4 => 'Adobe RGB',
-        },
-    },
-    0x0102 => {
-        Name => 'MinoltaQuality',
-        Writable => 'int16u',
-        PrintConv => {
-            0 => 'Raw',
-            1 => 'Superfine',
-            2 => 'Fine',
-            3 => 'Normal',
-            4 => 'Economy',
-            5 => 'Extra fine',
-        },
-    },
-    # (0x0103 is the same as 0x0102 above)
-    0x0103 => {
-        Name => 'MinoltaQuality',
-        Writable => 'int16u',
-        PrintConv => {
-            0 => 'Raw',
-            1 => 'Superfine',
-            2 => 'Fine',
-            3 => 'Normal',
-            4 => 'Economy',
-            5 => 'Extra fine',
-        },
-    },
     0x0104 => { #11
         Name => 'BodyFirmwareVersion',
         Writable => 'string',
@@ -200,7 +167,7 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     },
     0x0203 => { #6
         Name => 'BWMode',
-        Description => 'Black & White Mode',
+        Description => 'Black And White Mode',
         Writable => 'int16u',
         PrintConv => \%offOn,
     },
@@ -709,21 +676,24 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
 # the size, but also the count is 2 bytes short for the subdirectory itself
 # (presumably the Olympus programmers forgot about the 2-byte entry count at the
 # start of the subdirectory).  This mess is straightened out and these subdirs
-# are written properly when ExifTool rewrites the file.  (This problem has been
-# fixed in the new-style MakerNoteOlympus2 maker notes since a standard SubIFD
-# offset value is used.) - PH
+# are written properly when ExifTool rewrites the file.  Note that this problem
+# has been fixed by Olympus in the new-style IFD maker notes since a standard
+# SubIFD offset value is used.  As written by the camera, the old style
+# directories have format 'undef' or 'string', and the new style has format
+# 'ifd'.  However, some older versions of exiftool may have rewritten the new
+# style as 'int32u', so handle both cases. - PH
 #
     0x2010 => [ #PH
         {
             Name => 'Equipment',
-            Condition => 'not $$self{OlympusType2}',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::Equipment',
                 ByteOrder => 'Unknown',
             },
         },
         {
-            Name => 'Equipment2',
+            Name => 'EquipmentIFD',
             Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
             Flags => 'SubIFD',
             FixFormat => 'ifd',
@@ -736,14 +706,14 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     0x2020 => [ #PH
         {
             Name => 'CameraSettings',
-            Condition => 'not $$self{OlympusType2}',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::CameraSettings',
                 ByteOrder => 'Unknown',
             },
         },
         {
-            Name => 'CameraSettings2',
+            Name => 'CameraSettingsIFD',
             Groups => { 1 => 'MakerNotes' },
             Flags => 'SubIFD',
             FixFormat => 'ifd',
@@ -756,14 +726,14 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     0x2030 => [ #PH
         {
             Name => 'RawDevelopment',
-            Condition => 'not $$self{OlympusType2}',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::RawDevelopment',
                 ByteOrder => 'Unknown',
             },
         },
         {
-            Name => 'RawDevelopment2',
+            Name => 'RawDevelopmentIFD',
             Groups => { 1 => 'MakerNotes' },
             Flags => 'SubIFD',
             FixFormat => 'ifd',
@@ -776,14 +746,14 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     0x2031 => [ #11
         {
             Name => 'RawDev2',
-            Condition => 'not $$self{OlympusType2}',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::RawDevelopment2',
                 ByteOrder => 'Unknown',
             },
         },
         {
-            Name => 'RawDev2_2',
+            Name => 'RawDev2IFD',
             Groups => { 1 => 'MakerNotes' },
             Flags => 'SubIFD',
             FixFormat => 'ifd',
@@ -796,14 +766,14 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     0x2040 => [ #PH
         {
             Name => 'ImageProcessing',
-            Condition => 'not $$self{OlympusType2}',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::ImageProcessing',
                 ByteOrder => 'Unknown',
             },
         },
         {
-            Name => 'ImageProcessing2',
+            Name => 'ImageProcessingIFD',
             Groups => { 1 => 'MakerNotes' },
             Flags => 'SubIFD',
             FixFormat => 'ifd',
@@ -816,14 +786,14 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     0x2050 => [ #PH
         {
             Name => 'FocusInfo',
-            Condition => 'not ($$self{OlympusType2} or $$self{OlympusCAMER})',
+            Condition => '$format ne "ifd" and $format ne "int32u" and not $$self{OlympusCAMER}',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::FocusInfo',
                 ByteOrder => 'Unknown',
             },
         },
         {
-            Name => 'FocusInfo2',
+            Name => 'FocusInfoIFD',
             Condition => 'not $$self{OlympusCAMER}',
             Groups => { 1 => 'MakerNotes' },
             Flags => 'SubIFD',
@@ -841,80 +811,206 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
             Binary => 1,
         },
     ],
-    0x2100 => { #11
-        Name => 'Olympus2100',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+    0x2100 => [
+        { #11
+            Name => 'Olympus2100',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
         },
-    },
-    0x2200 => { #11
-        Name => 'Olympus2200',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+        { #PH
+            Name => 'Olympus2100IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
         },
-    },
-    0x2300 => { #11
-        Name => 'Olympus2300',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+    ],
+    0x2200 => [
+        { #11
+            Name => 'Olympus2200',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
         },
-    },
-    0x2400 => { #11
-        Name => 'Olympus2400',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+        { #PH
+            Name => 'Olympus2200IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
         },
-    },
-    0x2500 => { #11
-        Name => 'Olympus2500',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+    ],
+    0x2300 => [
+        { #11
+            Name => 'Olympus2300',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
         },
-    },
-    0x2600 => { #11
-        Name => 'Olympus2600',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+        { #PH
+            Name => 'Olympus2300IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
         },
-    },
-    0x2700 => { #11
-        Name => 'Olympus2700',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+    ],
+    0x2400 => [
+        { #11
+            Name => 'Olympus2400',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
         },
-    },
-    0x2800 => { #11
-        Name => 'Olympus2800',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+        { #PH
+            Name => 'Olympus2400IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
         },
-    },
-    0x2900 => { #11
-        Name => 'Olympus2900',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Olympus::FETags',
-            ByteOrder => 'Unknown',
+    ],
+    0x2500 => [
+        { #11
+            Name => 'Olympus2500',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
         },
-    },
+        { #PH
+            Name => 'Olympus2500IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
+        },
+    ],
+    0x2600 => [
+        { #11
+            Name => 'Olympus2600',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
+        },
+        { #PH
+            Name => 'Olympus2600IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
+        },
+    ],
+    0x2700 => [
+        { #11
+            Name => 'Olympus2700',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
+        },
+        { #PH
+            Name => 'Olympus2700IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
+        },
+    ],
+    0x2800 => [
+        { #11
+            Name => 'Olympus2800',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
+        },
+        { #PH
+            Name => 'Olympus2800IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
+        },
+    ],
+    0x2900 => [
+        { #11
+            Name => 'Olympus2900',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+            },
+        },
+        { #PH
+            Name => 'Olympus2900IFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::FETags',
+                ByteOrder => 'Unknown',
+                Start => '$val',
+            },
+        },
+    ],
     0x3000 => [
         { #6
             Name => 'RawInfo',
-            Condition => 'not $$self{OlympusType2}',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Olympus::RawInfo',
                 ByteOrder => 'Unknown',
             },
         },
         { #PH
-            Name => 'RawInfo2',
+            Name => 'RawInfoIFD',
             Groups => { 1 => 'MakerNotes' },
             Flags => 'SubIFD',
             FixFormat => 'ifd',
@@ -924,6 +1020,46 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
             },
         },
     ],
+    0x4000 => [ #PH
+        {
+            Name => 'MainInfo',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::Main',
+                ByteOrder => 'Unknown',
+            },
+        },
+        {
+            Name => 'MainInfoIFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::Main',
+                Start => '$val',
+            },
+        },
+    ],
+    0x5000 => [ #PH
+        {
+            Name => 'UnknownInfo',
+            Condition => '$format ne "ifd" and $format ne "int32u"',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::UnknownInfo',
+                ByteOrder => 'Unknown',
+            },
+        },
+        {
+            Name => 'UnknownInfoIFD',
+            Groups => { 1 => 'MakerNotes' },
+            Flags => 'SubIFD',
+            FixFormat => 'ifd',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Olympus::UnknownInfo',
+                Start => '$val',
+            },
+        },
+    ],
 );
 
 # TextInfo tags
@@ -994,6 +1130,10 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
             6 numbers: 0. Make, 1. Unknown, 2. Model, 3. Sub-model, 4-5. Unknown.  Only
             the Make, Model and Sub-model are used to determine the lens model
         },
+        # Have seen these values for the unknown numbers:
+        # 1: 0
+        # 4: 0, 2(Olympus lenses for which I have also seen 0 for this number)
+        # 5: 0, 16(new Lumix lenses)
         ValueConv => 'my @a = split(" ",$val); "$a[0] $a[2] $a[3]"',
         # set unknown values to zero when writing
         ValueConvInv => 'my @a=split(" ",$val); "$a[0] 0 $a[1] $a[2] 0 0"',
@@ -1035,19 +1175,23 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
             '1 6 0'  => 'Sigma 50-500mm F4.0-6.3 EX DG APO HSM RF', #6
             '1 7 0'  => 'Sigma 105mm F2.8 DG', #PH
             '1 8 0'  => 'Sigma 150mm F2.8 DG HSM', #PH
+            '1 16 0' => 'Sigma 24mm F1.8 EX DG Aspherical Macro', #PH
             '1 17 0' => 'Sigma 135-400mm F4.5-5.6 DG ASP APO RF', #11
             '1 18 0' => 'Sigma 300-800mm F5.6 EX DG APO', #11
             '1 20 0' => 'Sigma 50-500mm F4.0-6.3 EX DG APO HSM RF', #11
             '1 21 0' => 'Sigma 10-20mm F4.0-5.6 EX DC HSM', #11
             # Leica lenses (ref 11)
             '2 1 0'  => 'Leica D Vario Elmarit 14-50mm F2.8-3.5 Asph.',
+            '2 1 16' => 'Lumix G Vario 14-45mm F3.5-5.6 Asph. Mega OIS', #16
             '2 2 0'  => 'Leica D Summilux 25mm F1.4 Asph.',
+            '2 2 16' => 'Lumix G Vario 45-200mm F4-5.6 Mega OIS', #16
             '2 3 1'  => 'Leica D Vario Elmar 14-50mm F3.8-5.6 Asph.', #14 (L10 kit)
+            '2 3 16' => 'Lumix G Vario HD 14-140mm F4-5.8 Asph. Mega OIS', #16
             '2 4 0'  => 'Leica D Vario Elmar 14-150mm F3.5-5.6', #13
             '2 4 16' => 'Lumix G Vario 7-14mm F4 Asph.', #PH (E-P1 pre-production)
+            '2 5 16' => 'Lumix G 20mm F1.7 Asph.', #16
             '3 1 0'  => 'Leica D Vario Elmarit 14-50mm F2.8-3.5 Asph.',
             '3 2 0'  => 'Leica D Summilux 25mm F1.4 Asph.',
-            # (missing Panasonic Lumix G Vario HD 14-140mm/F4.0-5.8 Asph./Mega OIS)
         },
     },
     # apparently the first 3 digits of the lens s/n give the type (ref 4):
@@ -1278,7 +1422,11 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
         PrintConv => [{
             0 => 'AF Not Used',
             1 => 'AF Used',
-        }, {}], # (E-520 writes 2 values, 2nd is unknown - PH)
+        }],
+        # 2nd value written only by some models (u1050SW, u9000, uT6000, uT6010,
+        # uT8000, E-30, E-420, E-450, E-520, E-620, E-P1 and E-P2): - PH
+        # observed values when "AF Not Used": 0, 16
+        # observed values when "AF Used": 64, 96(face detect on), 256
     },
     0x303 => { #6
         Name => 'AFSearch',
@@ -1294,6 +1442,11 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
         Count => 64,
         PrintConv => 'Image::ExifTool::Olympus::PrintAFAreas($val)',
     },
+    0x307 => { #15
+        Name => 'AFFineTuneAdj',
+        Format => 'int16s',
+        Count => 3, # not sure what the 3 values mean
+    },
     0x400 => { #6
         Name => 'FlashMode',
         Writable => 'int16u',
@@ -1626,6 +1779,8 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
             '4 1280 0 0' => 'Light Tone',
             '5 1280 0 0' => 'Pin Hole',
             '6 1280 0 0' => 'Grainy Film',
+            '9 1280 0 0' => 'Diorama',
+            '10 1280 0 0' => 'Cross Process',
         },
     },
     0x600 => { #PH/4
@@ -2256,12 +2411,11 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
         Name => 'FaceDetect',
         Writable => 'int32u',
         Count => -1,
-        Notes => '2 or 3 values',
-        Relist => [ [0, 1], 2 ],
+        Notes => '1, 2 or 3 values',
         PrintConv => [{
-            '0 0' => 'Off',
-            '1 0' => 'On',
-        }, {}],
+            0 => 'Off',
+            1 => 'On',
+        }], # (other values are usually 0, but have seen 1 for 2nd value with FaceDetect On)
     },
     0x1201 => { #11
         Name => 'FaceDetectArea',
@@ -2457,7 +2611,7 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
         Writable => 'int16s',
         Notes => q{
             approximately Celsius for E-1, unknown calibration for E-3/410/420/510, and
-            always zero for E-300/330/400/500
+            always zero for E-300/330/400/500, divide by 10 for E-P1?
         },
     },
     0x1600 => { # ref http://fourthirdsphoto.com/vbb/showpost.php?p=107607&postcount=15
@@ -2676,6 +2830,13 @@ my %offOn = ( 0 => 'Off', 1 => 'On' );
     },
 );
 
+# Olympus unknown information tags
+%Image::ExifTool::Olympus::UnknownInfo = (
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+);
+
 # Tags found only in some FE models
 %Image::ExifTool::Olympus::FETags = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
