@@ -15,6 +15,7 @@
 #               8) http://graphics.cs.uni-sb.de/NMM/dist-0.9.1/Docs/Doxygen/html/mmreg_8h-source.html
 #               9) http://developers.videolan.org/vlc/vlc/doc/doxygen/html/codecs_8h-source.html
 #              10) http://wiki.multimedia.cx/index.php?title=TwoCC
+#              11) Andreas Winter (SCLive) private communication
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::RIFF;
@@ -23,7 +24,9 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.15';
+$VERSION = '1.20';
+
+sub ConvertTimeCode($);
 
 # type of current stream
 $Image::ExifTool::RIFF::streamType = '';
@@ -306,13 +309,26 @@ $Image::ExifTool::RIFF::streamType = '';
         Name => 'Tdat',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Tdat' },
     },
+    LIST_ncdt => { #PH (Nikon metadata)
+        Name => 'NikonData',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Nikon::AVI',
+            # define ProcessProc here so we don't need to load RIFF.pm from Nikon.pm
+            ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+        },
+    },
+    LIST_hydt => { #PH (Pentax metadata)
+        Name => 'PentaxData',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::AVI',
+            ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+        },
+    },
     JUNK => [
         {
             Name => 'OlympusJunk',
             Condition => '$$valPt =~ /^OLYMDigital Camera/',
-            SubDirectory => {
-                TagTable => 'Image::ExifTool::Olympus::AVI',
-            },
+            SubDirectory => { TagTable => 'Image::ExifTool::Olympus::AVI' },
         },
         {
             Name => 'CasioJunk',
@@ -330,7 +346,10 @@ $Image::ExifTool::RIFF::streamType = '';
             Name => 'RicohJunk',
             # the Ricoh Caplio GX stores sub-chunks in here
             Condition => '$$valPt =~ /^ucmt/',
-            SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::AVI' },
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Ricoh::AVI',
+                ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+            },
         },
         {
             Name => 'UnknownJunk',
@@ -384,6 +403,12 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::Info = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
+    NOTES => q{
+        RIFF INFO tags found in WAV audio and AVI video files.  Tags which are part
+        of the EXIF 2.2 specification (Tag ID's beginning with "I") have an
+        underlined Tag Name in the HTML version of this documentation.  Other tags
+        are found in AVI files generated by Sony Vegas video editing software.
+    },
     IARL => 'ArchivalLocation',
     IART => { Name => 'Artist',    Groups => { 2 => 'Author' } },
     ICMS => 'Commissioned',
@@ -416,12 +441,74 @@ $Image::ExifTool::RIFF::streamType = '';
     ISRC => 'Source',
     ISRF => 'SourceForm',
     ITCH => 'Technician',
+    # the following tags (TAPE->DTIM) are Sony Vegas AVI tags, and are
+    # also used by SCLive and Adobe Premier (ref 11)
+    TAPE => {
+        Name => 'TapeName',
+        Groups => { 2 => 'Video' },
+    },
+    TCOD => {
+        Name => 'StartTimeCode',
+        # this is the tape time code for the start of the video
+        Groups => { 2 => 'Video' },
+        ValueConv => '$val * 1e-7',
+        PrintConv => \&ConvertTimeCode,
+    },
+    TCDO => {
+        Name => 'EndTimeCode',
+        Groups => { 2 => 'Video' },
+        ValueConv => '$val * 1e-7',
+        PrintConv => \&ConvertTimeCode,
+    },
+    VMAJ => {
+        Name => 'VegasVersionMajor',
+        Groups => { 2 => 'Video' },
+    },
+    VMIN => {
+        Name => 'VegasVersionMinor',
+        Groups => { 2 => 'Video' },
+    },
+    CMNT => {
+        Name => 'Comment',
+        Groups => { 2 => 'Video' },
+    },
+    RATE => {
+        Name => 'Rate', #? (video? units?)
+        Groups => { 2 => 'Video' },
+    },
+    STAT => {
+        Name => 'Statistics',
+        Groups => { 2 => 'Video' },
+        # ("7318 0 3.430307 1", "0 0 3500.000000 1", "7 0 3.433228 1")
+        PrintConv => [
+            '"$val frames captured"',
+            '"$val dropped"',
+            '"Data rate $val"',
+            { 0 => 'Bad', 1 => 'OK' }, # capture OK?
+        ],
+    },
+    DTIM => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Groups => { 2 => 'Time' },
+        ValueConv => q{
+            my @v = split ' ', $val;
+            return undef unless @v == 2;
+            # get time in seconds
+            $val = 1e-7 * ($v[0] * 4294967296 + $v[1]);
+            # shift from Jan 1, 1601 to Jan 1, 1970
+            $val -= 134774 * 24 * 3600 if $val != 0;
+            return Image::ExifTool::ConvertUnixTime($val);
+        },
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
 );
 
 # Sub chunks of EXIF LIST chunk
 %Image::ExifTool::RIFF::Exif = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
+    NOTES => 'These tags are part of the EXIF 2.2 specification for WAV audio files.',
     ever => 'ExifVersion',
     erel => 'RelatedImageFile',
     etim => { Name => 'TimeCreated', Groups => { 2 => 'Time' } },
@@ -454,6 +541,10 @@ $Image::ExifTool::RIFF::streamType = '';
         Name => 'Stream',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Stream' },
     },
+    LIST_odml => {
+        Name => 'OpenDML',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::OpenDML' },
+    },
 );
 
 # Sub chunks of Tdat LIST chunk (ref PH)
@@ -470,7 +561,8 @@ $Image::ExifTool::RIFF::streamType = '';
     FIRST_ENTRY => 0,
     0 => {
         Name => 'FrameRate',
-        ValueConv => '$val ? 1e6 / $val : undef',
+        # (must use RawConv because raw value used in Composite tag)
+        RawConv => '$val ? 1e6 / $val : undef',
         PrintConv => 'int($val * 1000 + 0.5) / 1000',
     },
     1 => {
@@ -513,11 +605,22 @@ $Image::ExifTool::RIFF::streamType = '';
     ],
 );
 
+# Open DML tags (ref http://www.morgan-multimedia.com/download/odmlff2.pdf)
+%Image::ExifTool::RIFF::OpenDML = (
+    PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
+    GROUPS => { 2 => 'Video' },
+    dmlh => {
+        Name => 'ExtendedAVIHeader',
+        Flags => ['Binary','Unknown'],
+    },
+);
+
 %Image::ExifTool::RIFF::StreamHeader = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 2 => 'Video' },
     FORMAT => 'int32u',
     FIRST_ENTRY => 0,
+    PRIORITY => 0,  # so we get values from the first stream
     0 => {
         Name => 'StreamType',
         Format => 'string[4]',
@@ -529,18 +632,63 @@ $Image::ExifTool::RIFF::streamType = '';
             vids => 'Video',
         },
     },
-    1 => {
-        Name => 'Codec',
-        Format => 'string[4]',
-    },
+    1 => [
+        {
+            Name => 'AudioCodec',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "auds"',
+            Format => 'string[4]',
+        },
+        {
+            Name => 'VideoCodec',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "vids"',
+            Format => 'string[4]',
+        },
+        {
+            Name => 'Codec',
+            Format => 'string[4]',
+        },
+    ],
   # 2 => 'StreamFlags',
   # 3 => 'StreamPriority',
   # 3.5 => 'Language',
   # 4 => 'InitialFrames',
-  # 5 => 'Scale',
-  # 6 => 'Rate',
+    5 => [
+        {
+            Name => 'AudioSampleRate',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "auds"',
+            Format => 'rational64u',
+            ValueConv => '$val ? 1/$val : 0',
+            PrintConv => 'int($val * 100 + 0.5) / 100',
+        },
+        {
+            Name => 'VideoFrameRate',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "vids"',
+            Format => 'rational64u',
+            # (must use RawConv because raw value used in Composite tag)
+            RawConv => '$val ? 1/$val : undef',
+            PrintConv => 'int($val * 100 + 0.5) / 100',
+        },
+        {
+            Name => 'StreamSampleRate',
+            Format => 'rational64u',
+            ValueConv => '$val ? 1/$val : 0',
+            PrintConv => 'int($val * 100 + 0.5) / 100',
+        },
+    ],
   # 7 => 'Start',
-  # 8 => 'Length',
+    8 => [
+        {
+            Name => 'AudioSampleCount',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "auds"',
+        },
+        {
+            Name => 'VideoFrameCount',
+            Condition => '$Image::ExifTool::RIFF::streamType eq "vids"',
+        },
+        {
+            Name => 'StreamSampleCount',
+        },
+    ],
   # 9 => 'SuggestedBufferSize',
     10 => {
         Name => 'Quality',
@@ -584,7 +732,37 @@ $Image::ExifTool::RIFF::streamType = '';
             0 => 'FrameRate',
             1 => 'FrameCount',
         },
-        ValueConv => '$val[0] ? $val[1] / $val[0] : undef',
+        Desire => {
+            2 => 'VideoFrameRate',
+            3 => 'VideoFrameCount',
+        },
+        # this is annoying.  Apparently (although I couldn't verify this), FrameCount
+        # in the RIFF header includes multiple video tracks if they exist (ie. with the
+        # FujiFilm REAL 3D AVI's), but the video stream information isn't reliable for
+        # some cameras (ie. Olympus FE models), so use the video stream information
+        # only if the RIFF header duration is 2 to 3 times longer
+        RawConv => q {
+            my ($dur1, $dur2);
+            $dur1 = $val[1] / $val[0] if $val[0];
+            return $dur1 unless $val[2] and $val[3];
+            $dur2 = $val[3] / $val[2];
+            my $rat = $dur1 / $dur2;
+            return ($rat > 1.9 and $rat < 3.1) ? $dur2 : $dur1;
+        },
+        PrintConv => 'ConvertDuration($val)',
+    },
+    Duration2 => {
+        Name => 'Duration',
+        Require => {
+            0 => 'AvgBytesPerSec',
+            1 => 'FileSize',
+        },
+        Desire => {
+            # check FrameCount because this calculation only applies
+            # to audio-only files (ie. WAV)
+            2 => 'FrameCount',
+        },
+        RawConv => '($val[0] and not $val[2]) ? $val[1] / $val[0] : undef',
         PrintConv => 'ConvertDuration($val)',
     },
 );
@@ -616,6 +794,20 @@ sub ConvertRIFFDate($)
     return $val;
 }
 
+#------------------------------------------------------------------------------
+# Print time
+# Inputs: 0) time in seconds
+# Returns: time string
+sub ConvertTimeCode($)
+{
+    my $val = shift;
+    my $hr = int($val / 3600);
+    $val -= $hr * 3600;
+    my $min = int($val / 60);
+    $val -= $min * 60;
+    return sprintf("%d:%.2d:%05.2f", $hr, $min, $val);
+}
+
 #------------------------------------------------------------------------------
 # Process stream data
 # Inputs: 0) ExifTool object ref, 1) dirInfo reference, 2) tag table ref
@@ -683,6 +875,7 @@ sub ProcessChunks($$$)
     my $start = $$dirInfo{DirStart};
     my $size = $$dirInfo{DirLen};
     my $end = $start + $size;
+    my $base = $$dirInfo{Base};
 
     if ($exifTool->Options('Verbose')) {
         $exifTool->VerboseDir($$dirInfo{DirName}, 0, $size);
@@ -706,8 +899,16 @@ sub ProcessChunks($$$)
         if ($tagInfo) {
             if ($$tagInfo{SubDirectory}) {
                 # adjust base if necessary (needed for Ricoh maker notes)
-                my $newBase = $tagInfo->{SubDirectory}->{Base};
-                $baseShift = $newBase - $$dirInfo{Base} if defined $newBase;
+                my $newBase = $tagInfo->{SubDirectory}{Base};
+                if (defined $newBase) {
+                    # different than your average Base eval...
+                    # here we use an absolute $start address
+                    $start += $base;
+                    #### eval Base ($start)
+                    $newBase = eval $newBase;
+                    $baseShift = $newBase - $$dirInfo{Base};
+                    $start -= $base;
+                }
             } elsif (not $$tagInfo{Binary}) {
                 $val = substr($$dataPt, $start, $len);
                 $val =~ s/\0+$//;   # remove trailing nulls from strings
@@ -718,7 +919,7 @@ sub ProcessChunks($$$)
             DataPos => $$dirInfo{DataPos} - $baseShift,
             Start   => $start,
             Size    => $len,
-            Base    => $$dirInfo{Base} + $baseShift,
+            Base    => $base + $baseShift,
         );
         ++$len if $len & 0x01;  # must account for padding if odd number of bytes
         $start += $len;
@@ -767,13 +968,16 @@ sub ProcessRIFF($$)
         $exifTool->VPrint(0, "RIFF '$tag' chunk ($len bytes of data):\n");
         # stop when we hit the audio data or AVI index or AVI movie data
         # --> no more because Adobe Bridge stores XMP after this!!
-        #if ($tag eq 'data' or $tag eq 'idx1' or $tag eq 'LIST_movi') {
-        #    $exifTool->VPrint(0, "(end of parsing)\n");
-        #    last;
-        #}
+        # (so now we only do this on the FastScan option)
+        if (($tag eq 'data' or $tag eq 'idx1' or $tag eq 'LIST_movi') and
+            $exifTool->Options('FastScan'))
+        {
+            $exifTool->VPrint(0, "(end of parsing)\n");
+            last;
+        }
         # RIFF chunks are padded to an even number of bytes
         my $len2 = $len + ($len & 0x01);
-        if ($$tagTablePtr{$tag}) {
+        if ($$tagTablePtr{$tag} or ($verbose and $tag !~ /^(data|idx1|LIST_movi)$/)) {
             $raf->Read($buff, $len2) == $len2 or $err=1, last;
             $exifTool->HandleTag($tagTablePtr, $tag, $buff,
                 DataPt  => \$buff,
