@@ -87,7 +87,15 @@ my %xmpStruct = (
         version     => { },
     },
     Thumbnail => {
-        NAMESPACE => 'xapGImg',
+        NAMESPACE => 'xmpGImg',
+        height      => { },
+        width       => { },
+       'format'     => { },
+        image       => { },
+    },
+    PageInfo => {
+        NAMESPACE => 'xmpGImg',
+        PageNumber  => { Namespace => 'xmpTPg' }, # override default namespace for this element
         height      => { },
         width       => { },
        'format'     => { },
@@ -104,7 +112,7 @@ my %xmpStruct = (
         unit        => { },
     },
     Colorant => {
-        NAMESPACE => 'xapG',
+        NAMESPACE => 'xmpG',
         swatchName  => { },
         mode        => { },
         type        => { },
@@ -337,6 +345,19 @@ my %xmpStruct = (
         Sublocation  => { },
         WorldRegion  => { },
     },
+    # Microsoft Photo 1.2 structures
+    RegionInfo => {
+        NAMESPACE => 'MPRI',
+        Regions => {
+            Struct => 'Regions',
+            List => 'Bag',
+        },
+    },
+    Regions => {
+        NAMESPACE => 'MPReg',
+        Rectangle => { },
+        PersonDisplayName => { },
+    },
 );
 
 my $rdfDesc = 'rdf:Description';
@@ -379,7 +400,6 @@ my $pktCloseR =  "<?xpacket end='r'?>";
     }
     # update XMP tag tables
     my $mainTable = GetTagTable('Image::ExifTool::XMP::Main');
-    GenerateTagIDs($mainTable);
     foreach $mainTag (keys %$mainTable) {
         my $mainInfo = $mainTable->{$mainTag};
         next unless ref $mainInfo eq 'HASH' and $mainInfo->{SubDirectory};
@@ -398,7 +418,6 @@ my $pktCloseR =  "<?xpacket end='r'?>";
             $$table{NAMESPACE} = $ns;
         }
         $$table{WRITE_PROC} = \&WriteXMP;   # set WRITE_PROC for all tables
-        GenerateTagIDs($table);
         $table->{CHECK_PROC} = \&CheckXMP;  # add our write check routine
         foreach $tag (TagTableKeys($table)) {
             my $tagInfo = $$table{$tag};
@@ -481,24 +500,40 @@ sub FormatXMPDate($)
 
 #------------------------------------------------------------------------------
 # Check XMP values for validity and format accordingly
-# Inputs: 0) ExifTool object reference, 1) tagInfo hash reference,
-#         2) raw value reference
+# Inputs: 0) ExifTool object ref, 1) tagInfo hash ref, 2) raw value ref
 # Returns: error string or undef (and may change value) on success
 sub CheckXMP($$$)
 {
     my ($exifTool, $tagInfo, $valPtr) = @_;
-    # convert value from Latin if necessary
-    if ($exifTool->{OPTIONS}->{Charset} eq 'Latin' and $$valPtr =~ /[\x80-\xff]/) {
-        # convert from Latin to UTF-8
-        my $val = Image::ExifTool::Latin2Unicode($$valPtr,'n');
-        $$valPtr = Image::ExifTool::Unicode2UTF8($val,'n');
-    }
     my $format = $tagInfo->{Writable};
-    # if no format specified, value is a simple string
-    return undef unless $format and $format ne 'string';
+    # (if no format specified, value is a simple string)
+    if (not $format or $format eq 'string' or $format eq 'lang-alt') {
+        # convert value to UTF8 if necessary
+        if ($exifTool->{OPTIONS}->{Charset} ne 'UTF8') {
+            if ($$valPtr =~ /[\x80-\xff]/) {
+                # convert from Charset to UTF-8
+                my $val = $exifTool->Charset2Unicode($$valPtr, 'MM');
+                $$valPtr = Image::ExifTool::Unicode2UTF8($val,'n');
+            }
+        } else {
+            # translate invalid XML characters to "."
+            $$valPtr =~ tr/\0-\x08\x0b\x0c\x0e-\x1f/./;
+            # fix any malformed UTF-8 characters
+            if (FixUTF8($valPtr) and not $$exifTool{WarnBadUTF8}) {
+                $exifTool->Warn('Malformed UTF-8 character(s)');
+                $$exifTool{WarnBadUTF8} = 1;
+            }
+        }
+        return undef;   # success
+    }
     if ($format eq 'rational' or $format eq 'real') {
         # make sure the value is a valid floating point number
-        Image::ExifTool::IsFloat($$valPtr) or return 'Not a floating point number';
+        unless (Image::ExifTool::IsFloat($$valPtr) or
+            # allow 'inf' and 'undef' rational values
+            ($format eq 'rational' and ($$valPtr eq 'inf' or $$valPtr eq 'undef')))
+        {
+            return 'Not a floating point number' 
+        }
         if ($format eq 'rational') {
             $$valPtr = join('/', Image::ExifTool::Rationalize($$valPtr));
         }
@@ -515,8 +550,6 @@ sub CheckXMP($$$)
         my $newDate = FormatXMPDate($$valPtr);
         return "Invalid date/time (use YYYY:MM:DD HH:MM:SS[.SS][+/-HH:MM|Z])" unless $newDate;
         $$valPtr = $newDate;
-    } elsif ($format eq 'lang-alt') {
-        # nothing to do
     } elsif ($format eq 'boolean') {
         if (not $$valPtr or $$valPtr =~ /false/i or $$valPtr =~ /^no$/i) {
             $$valPtr = 'False';
@@ -554,7 +587,7 @@ sub SetPropertyPath($$;$$)
     my ($tagTablePtr, $tagID, $structPtr, $propList) = @_;
     my $table = $structPtr || $tagTablePtr;
     my $tagInfo = $$table{$tagID};
-    my $ns = $$table{NAMESPACE};
+    my $ns = $$tagInfo{Namespace} || $$table{NAMESPACE};
     # don't override existing main table entry if already set by a Struct
     return if not $structPtr and $$tagInfo{PropertyPath};
     $ns or warn("No namespace for $tagID\n"), return;
@@ -578,7 +611,7 @@ sub SetPropertyPath($$;$$)
     # set PropertyPath for all elements of this structure if necessary
     my $structName = $$tagInfo{Struct};
     if ($structName) {
-        my $struct = $xmpStruct{$structName} || 
+        my $struct = $xmpStruct{$structName} ||
                      $Image::ExifTool::UserDefined::xmpStruct{$structName};
         $struct or warn("No XMP $$tagInfo{Struct} structure!\n"), return;
         my $tag;
@@ -640,7 +673,7 @@ sub CaptureXMP($$$;$)
         # set flag so we don't write x:xmpmeta element
         $exifTool->{XMP_NO_XMPMETA} = 1;
         # add missing x:xmpmeta element and try again
-        unshift @$propList, 'x:xmpmeta';    
+        unshift @$propList, 'x:xmpmeta';
         CaptureXMP($exifTool, $propList, $val, $attrs);
     } else {
         $exifTool->{XMP_ERROR} = 'Improperly enclosed XMP property: ' . join('/',@$propList);
@@ -854,6 +887,109 @@ sub LimitXMPSize($$$$$$)
     return (\$extData, $guid);  # return extended XMP and its GUID
 }
 
+#------------------------------------------------------------------------------
+# Restore XMP structures in extracted information
+# Inputs: 0) ExifTool object ref
+sub RestoreStructure($)
+{
+    my $exifTool = shift;
+    my ($key, $nm, %structs, %var, $si);
+    my $ex = $$exifTool{TAG_EXTRA};
+    foreach $key (keys %{$$exifTool{TAG_INFO}}) {
+        $$ex{$key} or next;
+        my $structProps = $$ex{$key}{Struct} or next;
+        # preserve List-ness of List tags containing only a single value
+        if (@$structProps < 2) {
+            my $val = $$exifTool{VALUE}{$key};
+            $$exifTool{VALUE}{$key} = [ $val ] unless ref $val eq 'ARRAY';
+            next;
+        }
+        my $tagInfo = $$exifTool{TAG_INFO}{$key};
+        my $table = $$tagInfo{Table};
+        my $prop = shift @$structProps;
+        my $tag = $$prop[0];
+        # namespace is added to tag ID's in unknown table to avoid conflicts
+        if ($table eq \%Image::ExifTool::XMP::other) {
+            my $g1 = $$ex{$key}{G1} || $$tagInfo{Groups}{1};
+            my $ns = ($g1 and $g1=~/^XMP-(.*)/) ? $1 : 'unknown';
+            $tag = "$ns:$tag";
+        }
+        my $structInfo = $$table{$tag};
+        if ($structInfo) {
+            ref $structInfo eq 'HASH' or next;
+            unless ($$structInfo{SubDirectory}) {
+                $exifTool->Warn("[internal] $$tagInfo{Name} is not a SubDirectory!", 1);
+                next;
+            }
+        } else {
+            # create new entry in tag table for this structure
+            $structInfo = {
+                Name => ucfirst $$prop[0],
+                Groups => { 1 => $$ex{$key}{G1} || $$tagInfo{Groups}{1} },
+                SubDirectory => { },
+            };
+            Image::ExifTool::AddTagToTable($table, $tag, $structInfo);
+        }
+        # use structInfo ref for base key to avoid collisions
+        $tag = $structInfo;
+        # save structInfo ref and file order
+        $var{$structInfo} = [ $structInfo, $$exifTool{FILE_ORDER}{$key} ];
+        my $struct = \%structs;
+        my $oldStruct = $structs{$structInfo};
+        my $err;
+        for (;;) {
+            my $nextStruct = $$struct{$tag};
+            my $index = $$prop[1];
+            if (defined $index) {
+                $index = substr $index, 1;  # remove digit count
+                if ($nextStruct) {
+                    ref $nextStruct eq 'ARRAY' or $err = 1, last;
+                    $struct = $nextStruct;
+                } else {
+                    $struct = $$struct{$tag} = [ ];
+                }
+                $nextStruct = $$struct[$index];
+                if ($nextStruct) {
+                    ref $nextStruct eq 'HASH' or $err = 1, last;
+                    $struct = $nextStruct;
+                } elsif (@$structProps) {
+                    $struct = $$struct[$index] = { };
+                } else {
+                    $$struct[$index] = $exifTool->GetValue($key);
+                    last;
+                }
+            } else {
+                if ($nextStruct) {
+                    ref $nextStruct eq 'HASH' or $err = 1, last;
+                    $struct = $nextStruct;
+                } elsif (@$structProps) {
+                    $struct = $$struct{$tag} = { };
+                } else {
+                    $$struct{$tag} = $exifTool->GetValue($key);
+                    last;
+                }
+            }
+            $prop = shift @$structProps or last;
+            $tag = ucfirst $$prop[0];
+        }
+        if ($err) {
+            $exifTool->Warn("[internal] Error placing $$tagInfo{Name} in structure", 1);
+            unless ($oldStruct) {
+                delete $var{$structInfo};
+                delete $structs{$structInfo};
+            }
+        } else {
+            $exifTool->DeleteTag($key);
+        }
+    }
+    # save new structure tags
+    foreach $si (keys %structs) {
+        $key = $exifTool->FoundTag($var{$si}[0], '');
+        $$exifTool{VALUE}{$key} = $structs{$si};
+        $$exifTool{FILE_ORDER}{$key} = $var{$si}[1];
+    }
+}
+
 #------------------------------------------------------------------------------
 # Write XMP information
 # Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
@@ -873,8 +1009,8 @@ sub WriteXMP($$;$)
     my (%capture, %nsUsed, $xmpErr, $tagInfo, $about);
     my $changed = 0;
     my $xmpFile = (not $tagTablePtr);   # this is an XMP data file if no $tagTablePtr
-    # write XMP as preferred if this is an XMP file or a GIF file
-    my $preferred = $xmpFile || ($$exifTool{FILE_TYPE} and $$exifTool{FILE_TYPE} eq 'GIF');
+    # write XMP as preferred if this is an XMP, GIF or IND file
+    my $preferred = $xmpFile || ($$exifTool{FILE_TYPE} and $$exifTool{FILE_TYPE} =~ /^(GIF|IND)$/);
     my $verbose = $exifTool->Options('Verbose');
 #
 # extract existing XMP information into %capture hash
@@ -1150,12 +1286,12 @@ sub WriteXMP($$;$)
                 $path =~ s/(.*) 10/$1 11/ or warn "Internal error: no list index!\n", next;
             }
         }
-        
+
         # add new value(s) to %capture hash
         for (;;) {
             my $newValue = EscapeXML(shift @newValues);
             if ($$tagInfo{Resource}) {
-                $capture{$path} = [ '', { %attrs, 'rdf:resource' => $newValue } ]; 
+                $capture{$path} = [ '', { %attrs, 'rdf:resource' => $newValue } ];
             } else {
                 $capture{$path} = [ $newValue, \%attrs ];
             }
@@ -1403,9 +1539,10 @@ sub WriteXMP($$;$)
         }
         $newData .= ($$dirInfo{ReadOnly} ? $pktCloseR : $pktCloseW);
     }
-    # return empty data if no properties exist
-    $newData = '' unless %capture or $$dirInfo{InPlace};
-
+    # return empty data if no properties exist and this is allowed
+    unless (%capture or $xmpFile or $$dirInfo{InPlace} or $$dirInfo{NoDelete}) {
+        $newData = '';
+    }
     if ($xmpErr) {
         if ($xmpFile) {
             $exifTool->Error($xmpErr);
